#include "ofRecordMovie.h"


ofRecordMovie::ofRecordMovie (const char *filename){	
   	this->outputFile = strdup(filename);
	vWidth = -1;
	vHeight = -1;
	vBitRate = 0.0; 
	vFrameRate = 0.0;
	vCounter = 0;

	aSamplesPerSec = -1; 
	aChannels = -1;
	aBitRate = 0.0;

	vCodec = NULL;
	aCodec = NULL;
	mFormat = NULL;
   
	width	= ofGetWidth();
	height	= ofGetHeight();
	abortSignal = false;
	
	/* turn off texture use */
	screenpix.setUseTexture(false);
	
	e = new Encoder(outputFile);
}

ofRecordMovie::~ofRecordMovie (){
	cout << "ofRecordMovie destructor called." << endl;
	
	if(this->outputFile) free(this->outputFile);
	if(vCodec) free(vCodec);
	if(aCodec) free(aCodec);
	if(mFormat) free(mFormat);
	
 	if(e) e->close();
	delete e;
	

	
}

//---------------------------------------------------------------------------

ReturnCode ofRecordMovie::chooseVideoCodec(int width, int height, double bitRate, double frameRate, char *fcc)
{
  vWidth = width;
  vHeight = height;
  vBitRate = bitRate;
  vFrameRate = frameRate;
  if(vCodec) free(vCodec);
  if(fcc) vCodec = strdup(fcc);
  else vCodec = NULL;


	
  return OkCode;
}
ReturnCode ofRecordMovie::chooseAudioCodec(int samplesPerSec, int channels, double bitRate, char *fcc)
{
  aSamplesPerSec = samplesPerSec;
  aChannels = channels;
  aBitRate = bitRate;
  if(aCodec) free(aCodec);
  if(fcc) aCodec = strdup(fcc);
  else aCodec = NULL;
  
  return OkCode;
}

ReturnCode ofRecordMovie::chooseFormat(char *fcc)
{
  if(mFormat) free(mFormat);
  if(fcc)mFormat = strdup(fcc);
  else mFormat = NULL;
  e->chooseFormat(fcc);
  return OkCode;
}

ReturnCode ofRecordMovie::startRecord(void) {
	ReturnCode error;
	
	/* set video parameters */
	error = e->setVideoParameters(vWidth, vHeight, vBitRate, vFrameRate, vCodec);
	if(isError(error)) cout << "setVideoParameters Error =	" << getErrorMessage(error)<< endl;		

	/* set audio parameters */
	error =	e->setAudioParameters(aSamplesPerSec, aChannels, aBitRate, aCodec);
	if(isError(error)) cout << "setAudioParameters Error = " << getErrorMessage(error)<< endl;		
  
  	/* open encoder */
	error =	e->open();
	if(isError(error)) cout << "open encoder Error = " << getErrorMessage(error)<< endl;		

}

ReturnCode ofRecordMovie::encodeFrame()
{
	ReturnCode error = OkCode;

	if(abortSignal == false) 
	{
		/* get video frame */
		//cout << "encoding frame....";
		screenpix.grabScreen(0,0,width,height);
		if(screenpix.getPixels() != NULL) {
			e->addFrame((char *)screenpix.getPixels(), width, height);
			//cout << " Adding video frame: " << ++vCounter << " width=" << width << " height=" << height << endl;
		}
	
		/* Add AUDIO */
		//while(d->audioBuffer->count() > 0)
		//{			
			//add audio frame
			//cout << "Adding audio frame: " << ++aCounter << endl;
			//e->addAudioFrame(d->getAudioSamples(), d->getAudioSamplesSize(), d->getAudioChannelNumber(), d->getAudioSampleRate());
		//}

    } 
	
	return error;        
}

void ofRecordMovie::abort() {
	  abortSignal = true;
}

void ofRecordMovie::quit() {

	if(this->outputFile) free(this->outputFile);
	if(vCodec) free(vCodec);
	if(aCodec) free(aCodec);
	if(mFormat) free(mFormat);
	
 	if(e) e->close();
	delete e;

}
